<?php

/**
 * E2pdf Notification Helper
 * 
 * @copyright  Copyright 2017 https://e2pdf.com
 * @license    GPLv3
 * @version    1
 * @link       https://e2pdf.com
 * @since      0.00.01
 */
if (!defined('ABSPATH')) {
    die('Access denied.');
}

class Model_E2pdf_Notification extends Model_E2pdf_Model {

    /**
     * Add notification
     * 
     * @param string $type - Notification type
     * @param string $text - Notification text
     */
    public function add_notification($type, $text) {
        $notifications = get_transient('e2pdf_notifications');
        $notifications[] = array(
            'type' => $type,
            'text' => $text,
        );
        set_transient('e2pdf_notifications', $notifications);
    }

    /**
     * Get notifications
     * 
     * @return array - List of notifications
     */
    public function get_notifications() {
        $notifications = get_transient('e2pdf_notifications');

        if (!is_array($notifications)) {
            $notifications = array();
        }

        if (!get_option('e2pdf_hide_warnings', '0')) {
            if ($this->helper->get('license')->get('status') == 'pre_expired') {
                $message = sprintf(__('Your E2Pdf License Key will expire at <strong>%s</strong>', 'e2pdf'), $this->helper->get('license')->get('expire'));
                if (current_user_can('manage_options') || current_user_can('e2pdf_license')) {
                    $message .= ' | ' . sprintf(__('<a class="e2pdf-link" target="_blank" href="%s">Renew License Key »</a>', 'e2pdf'), 'https://e2pdf.com/checkout/license/renew/' . get_option('e2pdf_license'));
                }
                array_unshift($notifications, array(
                    'type' => 'notice',
                    'text' => $message
                ));
            }

            if ($this->helper->get('license')->get('status') == 'expired') {
                $message = __('Your E2Pdf License Key has expired', 'e2pdf');
                if (current_user_can('manage_options') || current_user_can('e2pdf_license')) {
                    $message .= ' | ' . sprintf(__('<a class="e2pdf-link" target="_blank" href="%s">Renew License Key »</a>', 'e2pdf'), 'https://e2pdf.com/checkout/license/renew/' . get_option('e2pdf_license'));
                }
                array_unshift($notifications, array(
                    'type' => 'error',
                    'text' => $message
                ));
            }

            if ($this->helper->get('license')->get('type') == 'FREE' && $this->helper->get('page') == 'e2pdf-templates') {
                array_unshift($notifications, array(
                    'type' => 'notice',
                    'text' => sprintf(__('You are using E2Pdf FREE License Type | Up to 1 Page and up to 1 Template allowed | <a class="e2pdf-link" target="_blank" href="%s">Upgrade License Key »</a>', 'e2pdf'), 'https://e2pdf.com/price')
                ));
            }
        }

        if ($this->helper->get('license')->get('error')) {
            foreach ($notifications as $key => $notify) {
                if ($notify['type'] === 'error' && $notify['text'] === $this->helper->get('license')->get('error')) {
                    unset($notifications[$key]);
                }
            }
            if ($this->helper->get('license')->get('error') === 'License Key does not match this site. Please correct License Key to continue usage.') {
                $message = __('E2Pdf License Key does not match this site', 'e2pdf');
                if (current_user_can('manage_options') || current_user_can('e2pdf_license')) {
                    $message .= ' | ' . sprintf(__('<a class="e2pdf-link" id="e2pdf-restore-license-key"  href="javascript:void(0)" _wpnonce="%s">Restore License Key »</a>', 'e2pdf'), wp_create_nonce('e2pdf_license'));
                }
            } else {
                $message = $this->helper->get('license')->get('error');
            }
            array_unshift($notifications, array(
                'type' => 'error',
                'text' => $message
            ));
        }

        set_transient('e2pdf_notifications', array());
        return $notifications;
    }
}
